# -*- coding: utf-8 -*-
# 
# django-granadilla
# Copyright (C) 2009-2012 Bolloré telecom
# See AUTHORS file for a full list of contributors.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import unicode_literals

import base64
import colorama
import datetime
import django
import inspect
import logging
import os
import os.path
import termios
import time
import re
import sys

import zxcvbn

if django.VERSION[:2] >= (1, 7):
    django.setup()

from .conf import settings
from . import models

PY2 = sys.version_info[0] == 2

if PY2:
    def force_text(txt):
        return txt.decode('utf-8')
else:
    def force_text(txt):
        return txt

# configure logging
logger = logging.getLogger()
handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter('%(asctime)s %(levelname)-8s %(message)s'))
logger.addHandler(handler)
logger.setLevel(logging.DEBUG)


def command(fun):
    """Decorator that marks a method as "publicly callable".

    Usage:

        >>> @command
        ... def foo(self):
        ...     pass
    """
    fun.is_command = True
    return fun


class CLI(object):

    PASSWORD_MIN_SCORE = 3

    def _write(self, txt, args, color=colorama.Fore.RESET, target=sys.stdout):
        txt = txt % args
        txt = '%s%s%s\n' % (color, txt, colorama.Fore.RESET)
        if PY2:
            target.write(txt.encode('utf-8'))
        else:
            target.write(txt)

    def display(self, txt, *args):
        self._write(txt, args)

    def success(self, txt, *args):
        self._write(txt, args, color=colorama.Fore.GREEN)

    def warn(self, txt, *args):
        self._write(txt, args, color=colorama.Fore.YELLOW)

    def error(self, txt, *args):
        self._write(txt, args, color=colorama.Fore.RED, target=sys.stderr)

    def change_password(self, user):
        password = None

        blacklist = [
            user.username,
            user.first_name,
            user.last_name,
        ]

        while password is None:
            password = self._get_good_password(blacklist)

        user.set_password(password)

    def change_device_password(self, device):
        password = None

        blacklist = [
            device.device_owner,
            device.device_name,
        ]

        while password is None:
            password = self._get_good_password(blacklist)

        device.set_password(password)

    def _get_good_password(self, blacklist):
        password1 = self.grab("Password: ", True)
        password2 = self.grab("Password (again): ", True)
        if password2 != password1:
            self.error("Passwords do not match, try again.")
            return None

        check = zxcvbn.password_strength(password1, blacklist)
        if check['score'] < self.PASSWORD_MIN_SCORE:
            self.error("Password is too weak (bruteforce: %s)", check['crack_time_display'])
            return None

        self.success("Password is strong enough (bruteforce: %s)", check['crack_time_display'])
        return password1

    def grab(self, prompt, password=False):

        if password:
            fd = sys.stdin.fileno()
            old = termios.tcgetattr(fd)
            new = termios.tcgetattr(fd)
            new[3] = new[3] & ~termios.ECHO
            try:
                termios.tcsetattr(fd, termios.TCSADRAIN, new)
                passwd = force_text(raw_input(prompt))
            finally:
                termios.tcsetattr(fd, termios.TCSADRAIN, old)
            sys.stdout.write("\n")
            return passwd

        else:
            sys.stdout.write(prompt)
            return force_text(sys.stdin.readline()).strip()

    def fill_object(self, obj, fields):
        for key in fields:
            field = obj._meta.get_field(key)
            name = key.replace("_", " ").title()
            default = getattr(obj, key)
            new_value = ''

            if default:
                new_value = self.grab("%s [%s]: " % (name, default))
            else:
                while not len(new_value):
                    new_value = self.grab("%s: " % (name))

            if new_value:
                setattr(obj, key, new_value)

    @command
    def addgroup(self, groupname):
        """
        Create a new group.
        """
        # get gid
        gids = models.LdapGroup.objects.values_list('gid', flat=True)
        if gids:
            gid = max(gids) + 1
        else:
            gid = 10000

        # create group
        group = models.LdapGroup()
        group.name = groupname
        group.gid = gid
        group.save()

    @command
    def adduser(self, username):
        """
        Create a new user.
        """
        # create user
        users = models.LdapUser.objects.all()
        if users:
            id = max([ x.uid for x in users ]) + 1
        else:
            id = 10000

        # prompt for information
        user = models.LdapUser()
        user.username = username
        user.uid = id
        self.fill_object(user, ['first_name', 'last_name'])
        for key in ['full_name', 'gecos', 'group', 'email', 'home_directory', 'login_shell']:
            setattr(user, key, user.defaults(key))
        self.fill_object(user, ['email'])
        self.change_password(user)

        # save user
        user.save()

    @command
    def addusergroup(self, username, groupname):
        """Add user <username> to group <groupname>."""
        user = models.LdapUser.objects.get(username=username)
        group = models.LdapGroup.objects.get(name=groupname)
        if not user.username in group.usernames:
            group.usernames.append(user.username)
            group.save()

        if settings.GRANADILLA_USE_ACLS:
            try:
                acl = models.LdapAcl.objects.get(name=groupname)
                if not user.dn in acl.members:
                    acl.members.append(user.dn)
                    acl.save()
            except models.LdapAcl.DoesNotExist:
                acl = models.LdapAcl()
                acl.name = groupname
                acl.members = [ user.dn ]
                acl.save()

    @command
    def catgroup(self, groupname):
        """
        Display a group's details.
        """
        group = models.LdapGroup.objects.get(name=groupname)
        self.display("dn: %s", group.dn)
        for field in group._meta.fields:
            if field.db_column:
                val = getattr(group, field.name, None)
                if val:
                    self.display("%s: %s", field.db_column, val)

    @command
    def catuser(self, username):
        """
        Display a user's details.
        """
        user = models.LdapUser.objects.get(username=username)
        self.display("dn: %s", user.dn)
        for field in user._meta.fields:
            if field.db_column and field.db_column != "jpegPhoto":
                val = getattr(user, field.name, None)
                if val:
                    self.display("%s: %s", field.db_column, val)

    @command
    def delgroup(self, groupname):
        """
        Delete the given group.
        """
        group = models.LdapGroup.objects.get(name=groupname)
        self.warn("Deleting group %s", group.dn)
        group.delete()
        if settings.GRANADILLA_USE_ACLS:
            try:
                models.LdapAcl.objects.get(name=groupname).delete()
            except models.LdapAcl.DoesNotExist:
                pass

    @command
    def delusergroup(self, username, groupname):
        """Remove a user from a group."""
        user = models.LdapUser.objects.get(username=username)
        group = models.LdapGroup.objects.get(name=groupname)

        self._delusergroup(user, group)

    def _delusergroup(self, user, group):
        if user.username in group.usernames:
            self.warn("Removing %s from group %s", user.username, group.name)
            group.usernames = [ x for x in group.usernames if x != user.username ]
            group.save()

        if settings.GRANADILLA_USE_ACLS:
            try:
                acl = models.LdapAcl.objects.get(name=group.name)
                if user.dn in acl.members:
                    acl.members = [ x for x in acl.members if x != user.dn ]
                    acl.save()
            except models.LdapAcl.DoesNotExist:
                pass


    @command
    def deluser(self, username):
        """
        Delete the given user.
        """
        user = models.LdapUser.objects.get(username=username)

        # delete user
        for group in models.LdapGroup.objects.all():
            self._delusergroup(user, group)

        self.warn("Removing user %s", user.dn)
        user.delete()

    @command
    def init(self):
        """
        Initialise the LDAP directory.
        """
        # create organizational units
        dns = [
            settings.GRANADILLA_USERS_DN,
            settings.GRANADILLA_EXTERNAL_USERS_DN,
            settings.GRANADILLA_DEVICES_DN,
            settings.GRANADILLA_GROUPS_DN,
            settings.GRANADILLA_SERVICES_DN,
        ]
        if settings.GRANADILLA_USE_ACLS:
            dns += [settings.GRANADILLA_ACLS_DN]

        for dn in dns:
            # FIXME: this may not be accurate depending on the DN
            name = dn.split(",")[0].split("=")[1]
            try:
                ou = models.LdapOrganizationalUnit.objects.get(name=name)
            except models.LdapOrganizationalUnit.DoesNotExist:
                ou = models.LdapOrganizationalUnit()
                ou.name = name
                ou.save()

        # create default group
        try:
            models.LdapGroup.objects.get(name=settings.GRANADILLA_USERS_GROUP)
        except:
            self.addgroup(settings.GRANADILLA_USERS_GROUP)

    @command
    def lsgroups(self):
        """Print the list of groups"""
        for group in models.LdapGroup.objects.all():
            self.display(group.name)

    @command
    def lsgroup(self, groupname):
        """
        Print the members of one group
        """
        members = models.LdapGroup.objects.get(name=groupname).usernames
        others = [ x.username for x in models.LdapUser.objects.all() if not x.username in members ]
        self.display("members:")
        for member in sorted(members):
            self.display("  %s", member)
        self.display("")
        self.display("non-members:")
        for other in sorted(others):
            self.display("  %s", other)

    @command
    def lsuser(self):
        """
        Print the list of users.
        """
        self.display("%20s%50s%20s", "username", "Email", "Password last set")
        for user in models.LdapUser.objects.order_by('username'):
            if user.samba_pwdlastset > time.time() - 3 * 365 * 24 * 60 * 60:
                pwd_last_set = datetime.date.fromtimestamp(user.samba_pwdlastset).strftime('%d %b %Y')
            else:
                pwd_last_set = "long ago"
            self.display("%20s%50s%20s", user.username, user.email, pwd_last_set)

    @command
    def lspasswd(self):
        """
        Print the list of passwords.
        """
        for user in models.LdapUser.objects.order_by('username'):
            self.display(user.password)

    @command
    def lsjohnpasswd(self):
        """
        Print the list of password formated for john
        """
        password_re = re.compile(r'^{\w+}([A-Za-z0-9/+=]+)$')  # {MD5}uihGYUGE==
        for user in models.LdapUser.objects.order_by('username'):
            match = password_re.match(user.password)
            if not match:
                self.warn("Password of user %s doesn't match {<ALGO>}<hash> format.", user.username)
                continue
            password = match.groups()[0]
            john_password = base64.b16encode(base64.b64decode(password)).lower()
            self.display("%s:%s", user.username, john_password)

    @command
    def lsusergroups(self, username):
        """Print the groups a user belongs to."""
        user = models.LdapUser.objects.get(username=username)
        self.display("Groups for %s (%s):\n", user.username, user.email)

        for group in models.LdapGroup.objects.order_by('name'):
            if user.username in group.usernames:
                self.display(group.name)

    @command
    def moduser(self, username, attr, value):
        """
        Modify an attribute for a user.
        """
        user = models.LdapUser.objects.get(username=username)
        for field in user._meta.fields:
            if field.db_column == attr:
                setattr(user, field.name, value)
                user.save()
                return
        raise Exception("Unnown field %s" % attr)

    @command
    def passwd(self, username):
        """
        Change the given user's password.
        """
        user = models.LdapUser.objects.get(username=username)
        self.change_password(user)
        user.save()

    @command
    def service_list(self):
        """
        Print the list of service accounts.
        """
        for account in models.LdapServiceAccount.objects.order_by('username'):
            self.display("%-20s %s", account.username, account.description.replace('\n', '  '))

    @command
    def service_add(self, username):
        """
        Add a new service account.
        """
        account = models.LdapServiceAccount()
        account.username = username
        self.fill_object(account, ['description'])
        self.change_password(account)

        account.save()

    @command
    def service_mod(self, username, attr, value):
        """
        Modify an attribute for a service.
        """
        account = models.LdapServiceAccount.objects.get(username=username)
        for field in account._meta.fields:
            if field.db_column == attr:
                setattr(account, field.name, value)
                account.save()
                return
        raise Exception("Unknown field %s" % attr)

    @command
    def service_del(self, username):
        """
        Delete a service account.
        """
        account = models.LdapServiceAccount.objects.get(username=username)
        self.warn("Deleting service %s", account.dn)
        account.delete()

    @command
    def service_passwd(self, username):
        """
        Change the password of a service.
        """
        account = models.LdapServiceAccount.objects.get(username=username)
        self.change_password(account)
        account.save()

    @command
    def extuser_list(self):
        """
        Print the list of extuser accounts.
        """
        for account in models.LdapExternalUser.objects.order_by('email'):
            self.display("%-20s %s", account.email, account.full_name)

    @command
    def extuser_add(self, email):
        """
        Add a new extuser account.
        """
        account = models.LdapExternalUser()
        account.email = email
        self.fill_object(account, [
            'first_name',
            'last_name',
        ])

        account.save()

    @command
    def extuser_addingroup(self, email, groupname):
        """
        Add an extuser in a group.
        """
        account = models.LdapExternalUser.objects.get(email=email)
        group = models.LdapGroup.objects.get(name=groupname)

        if settings.GRANADILLA_USE_ACLS:
            try:
                acl = models.LdapAcl.objects.get(name=groupname)
            except models.LdapAcl.DoesNotExist:
                acl = models.LdapAcl()
                acl.name = groupname
                acl.members = []

            if account.dn not in acl.members:
                acl.members.append(account.dn)
            acl.save()

    @command
    def extuser_lsgroups(self, email):
        """
        List the groups of an extuser.
        """
        account = models.LdapExternalUser.objects.get(email=email)

        for acl in models.LdapAcl.objects.order_by('name'):
            if account.dn in acl.members:
                self.display(acl.name)

    @command
    def extuser_delfromgroup(self, email, groupname):
        """
        Remove an extuser from a group.
        """
        account = models.LdapExternalUser.objects.get(email=email)
        group = models.LdapGroup.objects.get(name=groupname)

        self.warn("Removing %s from group %s", account.email, group.name)

        if settings.GRANADILLA_USE_ACLS:
            try:
                acl = models.LdapAcl.objects.get(name=groupname)
            except models.LdapAcl.DoesNotExist:
                return

            if account.dn in acl.members:
                acl.members = [ dn for dn in acl.members if dn != account.dn ]
                acl.save()

    @command
    def extuser_mod(self, email, attr, value):
        """
        Modify an attribute for a extuser.
        """
        account = models.LdapExternalUser.objects.get(email=email)
        for field in account._meta.fields:
            if field.db_column == attr:
                setattr(account, field.name, value)
                account.save()
                return
        raise Exception("Unknown field %s" % attr)

    @command
    def extuser_del(self, email):
        """
        Delete a extuser account.
        """
        account = models.LdapExternalUser.objects.get(email=email)
        self.warn("Deleting extuser %s", account.dn)
        account.delete()

    @command
    def device_list(self):
        """
        Print the list of devices and their owner.
        """
        for device in models.LdapDevice.objects.order_by('device_fullname'):
            self.display("%s", device.device_fullname)

    @command
    def device_add(self, username):
        """
        Add a new device.
        """
        device = models.LdapDevice()
        user = models.LdapUser.objects.get(username=username)
        device.device_username = username
        device.device_owner = user.dn
        device.device_name = self.grab("Device name: ", False) 
        device.device_fullname = username + "_" + device.device_name
        self.change_device_password(device)
        device.save()

    @command
    def device_password(self, username, device_name):
        """
        Change the password of the device.
        """
        device = models.LdapDevice.objects.get(
                    device_fullname=username + "_" + device_name)
        self.change_device_password(device)
        device.save()

    @command
    def device_del(self, username, device_name):
        """
        Delete a device.
        """
        device = models.LdapDevice.objects.get(
                        device_fullname=username + "_" + device_name)
        self.warn("Deleting device %s", device_name)
        device.delete()

    @command
    def help(self):
        """
        Display a help message.
        """
        cmdhelp = []
        for cmd in dir(self):
            func = getattr(self, cmd)
            if not getattr(func, 'is_command', False):
                continue

            bits = [cmd]
            bits.extend(["<%s>" % arg for arg in inspect.getargspec(func)[0][1:] ])
            cmdhelp.append("%s%s" % (" ".join(bits).ljust(50), func.__doc__.strip()))

        self.display("""Usage: %s <command> [arguments..]

Commands:
%s
""", os.path.basename(sys.argv[0]), "\n".join(cmdhelp))


    def main(self, argv):
        if len(argv) < 2:  # No command
            self.help()
            return 1

        cmd = argv[1]
        args = argv[2:]
        meth = getattr(self, cmd, None)
        if meth is None or not getattr(meth, 'is_command', False):
            self.error("Unknown command %s", cmd)
            self.help()
            return 1

        try:
            meth(*args)
        except models.LdapUser.DoesNotExist:
            self.error("The requested user does not exist.")
            return 2
        except (models.LdapAcl.DoesNotExist, models.LdapGroup.DoesNotExist):
            self.error("The requested group does not exist.")
            return 2
        except models.LdapServiceAccount.DoesNotExist:
            self.error("The requested service account does not exist.")
            return 2
        except models.LdapExternalUser.DoesNotExist:
            self.error("The requested external user does not exist.")
            return 2


def launch_cli():
    """Main 'cli' entry point."""
    cli = CLI()
    retcode = cli.main(sys.argv)
    sys.exit(retcode)


if __name__ == "__main__":
    launch_cli()



# -*- coding: utf-8 -*-
# 
# django-granadilla
# Copyright (C) 2009-2012 Bolloré telecom
# See AUTHORS file for a full list of contributors.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import unicode_literals
import re

from . import models
from django import forms
from django.core.files.uploadedfile import SimpleUploadedFile
from django.utils.translation import ugettext as _


class LdapContactForm(forms.ModelForm):
    postal_address = forms.CharField(required=False, label=_("Postal address"), widget=forms.Textarea)

    def __init__(self, *args, **kwargs):
        if 'base_dn' in kwargs:
            self.__base_dn = kwargs.pop('base_dn')
        else:
            self.__base_dn = None
        super(LdapContactForm, self).__init__(*args, **kwargs)

    def save(self, commit=True):
        contact = super(LdapContactForm, self).save(False)
        if self.__base_dn:
            contact.base_dn = self.__base_dn
        if commit:
            contact.save()
        return contact

    class Meta:
        model = models.LdapContact
        exclude = ("dn", "photo", "full_name")


class LdapDeviceForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        if 'base_dn' in kwargs:
            self._base_dn = kwargs.pop('base_dn')
        else:
            self._base_dn = None

        self._username = kwargs.pop('username')
        self.tmppwd = kwargs.pop('tmppwd')

        super(LdapDeviceForm, self).__init__(*args, **kwargs)

    def is_valid(self):
        if super(LdapDeviceForm, self).is_valid():    
            # Check if the device already exists
            device_fullname = self._username + '_' + \
                        self.data['device_name']
            try:
                if models.LdapDevice.objects.get(device_fullname=device_fullname):
                    self.errors['devicenameerror'] = "A device with \
                            this name already exists"
                    return False
            except models.LdapDevice.DoesNotExist:
                return True
        return False

    def save(self, commit=True):
        device = super(LdapDeviceForm, self).save(False)

        device.device_username = self._username
        user = models.LdapUser.objects.get(username=device.device_username)
        device.device_owner = user.dn
        device.device_fullname = device.device_username \
                         + '_' + self.data['device_name']

        # Initialize with random password because the field is necessary
        device.set_password(self.tmppwd)
        self.tmppwd = ""

        if self._base_dn:
            device.base_dn = self._base_dn
        if commit:
            device.save()
        return device

    class Meta:
        model = models.LdapDevice
        exclude = ("password", "dn", "device_username", "device_owner", "device_fullname")


class LdapUserForm(forms.ModelForm):
    new_photo = forms.ImageField(required=False, label=_("Photo"))

    def save(self, commit=True):
        contact = super(LdapUserForm, self).save(False)
        photo = self.cleaned_data['new_photo']
        if hasattr(photo, 'read'):
            contact.photo = photo.read()
        if commit:
            contact.save()
        return contact

    class Meta:
        model = models.LdapUser
        fields = ('phone', 'mobile_phone', 'internal_phone', 'new_photo')



# -*- coding: utf-8 -*-
# 
# django-granadilla
# Copyright (C) 2009-2012 Bolloré telecom
# See AUTHORS file for a full list of contributors.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import unicode_literals

import time

from .conf import settings
from django.contrib.auth.decorators import login_required
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.http import HttpResponse, Http404, HttpResponseRedirect, HttpResponseNotModified
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.utils.http import http_date
from django.views.static import was_modified_since
from django.views import generic as generic_views

from granadilla.templatetags.granadilla_tags import granadilla_media
from granadilla.forms import LdapContactForm, LdapDeviceForm, LdapUserForm
from . import models
from . import vcard

def can_write(user, entry):
    """
    Check whether a user can write an LDAP entry.
    """
    can_edit = user.username == entry.username or user.is_superuser
    for group in user.groups.all():
        if group.name in settings.GRANADILLA_ADMIN_GROUPS:
            return True
    return can_edit

def get_contacts(user):
    base_dn = "ou=%s,%s" % (user.username, settings.GRANADILLA_CONTACTS_DN)
    return models.LdapContact.scoped(base_dn)

def get_devices():
    base_dn = "%s" % (settings.GRANADILLA_DEVICES_DN)
    return models.LdapDevice.scoped(base_dn)

def get_random_password(length=32, allowed_chars='abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'):
        import random
        try:
            random = random.SystemRandom()
        except NotImplementedError:
            pass
        return ''.join([random.choice(allowed_chars) for i in range(length)])
    
def user_vcard(user):
    """Return the vCard for a contact."""
    card = vcard.VCard()
    card['kind'] = 'individual'
    card['names'] = [
        [user.first_name],
        [user.last_name],
        [],  # Additional names
        [],  # Honorific prefixes
        [],  # Honorific suffixes
    ]
    card['full_name'] = user.full_name
    card['email'] = user.email
    card['org'] = getattr(user, 'organization', '')
    card['phone'] = user.phone
    card['cell'] = user.mobile_phone

    # send response
    response = HttpResponse(card.render_bytes(), "text/x-vcard; charset=utf-8")
    response['Content-Disposition'] = "attachment; filename=%s.vcf" % user.pk.replace(' ', '')
    return response
    
@login_required
def index(request, template_name='granadilla/facebook.html'):
    return group(request, pk=settings.GRANADILLA_USERS_GROUP)


@login_required
def contact_card(request, contact_id):
    contact = get_contacts(request.user).objects.get(pk=contact_id)
    return user_vcard(contact)


class ContactCreate(generic_views.CreateView):
    model = models.LdapContact
    template_name = 'granadilla/contact.html'

    form_class = LdapContactForm

    def get_form_kwargs(self):
        base_dn = 'ou=%s,%s' % (self.request.user.username, settings.GRANADILLA_CONTACTS_DN)
        kwargs = super(ContactCreate, self).get_form_kwargs()
        kwargs['base_dn'] = base_dn
        return kwargs

    def get_success_url(self):
        return reverse(contact_list)

contact_create = login_required(ContactCreate.as_view())


class ContactUpdate(generic_views.UpdateView):
    model = models.LdapContact
    template_name = 'granadilla/contact.html'
    form_class = LdapContactForm

    def get_queryset(self):
        return get_contacts(self.request.user).objects.all()

contact = login_required(ContactUpdate.as_view())


class ContactDelete(generic_views.DeleteView):
    model = models.LdapContact
    template_name = 'granadilla/contact_delete.html'

    def get_queryset(self):
        return get_contacts(self.request.user).objects.all()

    def get_success_url(self):
        return reverse(contact_list)


contact_delete = login_required(ContactDelete.as_view())


class ContactListView(generic_views.ListView):
    model = models.LdapContact
    template_name = 'granadilla/contact_list.html'

    def get_queryset(self):
        return get_contacts(self.request.user).objects.all()


contact_list = login_required(ContactListView.as_view())


class DeviceCreate(generic_views.CreateView):
    """
    Creating a new device
    """
    model = models.LdapDevice
    template_name = 'granadilla/device.html'

    form_class = LdapDeviceForm

    def get_form_kwargs(self):
        base_dn = '%s' % (settings.GRANADILLA_DEVICES_DN,)
        kwargs = super(DeviceCreate, self).get_form_kwargs()
        kwargs['base_dn'] = base_dn
        kwargs['username'] = self.request.user.username
        kwargs['tmppwd'] = get_random_password()
        return kwargs

    def get_success_url(self):
        return reverse(device_list)

device_create = login_required(DeviceCreate.as_view())

class DevicePassword(generic_views.DetailView):
    """
    Generating a new password
    """
    model = models.LdapDevice
    template_name = 'granadilla/device_password.html'

    def get_object(self, queryset=None):
        try:
            device = get_devices().objects.get(device_fullname=self.kwargs['device_fullname'])
        except models.LdapDevice.DoesNotExist:
            return None

        user = self.request.user
        if user.username == device.device_username:
            # Change the password and display it using tmppwd 
            device.tmppwd = get_random_password()
            device.set_password(device.tmppwd)
            device.save()
            return device
        else:
            return None

    def post(self, *args, **kwargs):
        return HttpResponseRedirect(reverse(device_list))

device_password = login_required(DevicePassword.as_view())


class DeviceDelete(generic_views.DeleteView):
    """
    Deleting a device
    """
    model = models.LdapDevice
    template_name = 'granadilla/device_delete.html'

    slug_field = 'device_fullname'

    def get_object(self, queryset=None):
        try:
            device = get_devices().objects.get(device_fullname=self.kwargs['device_fullname'])
        except models.LdapDevice.DoesNotExist:
            return None

        user = self.request.user
        if user.is_superuser or user.username == device.device_username:
            return device
        else:
            return None

    def get_success_url(self):
        return reverse(device_list)


device_delete = login_required(DeviceDelete.as_view())


class DeviceAttrView(generic_views.ListView):
    """
    Displaying the device's attribute
    """
    model = models.LdapDevice
    template_name = 'granadilla/device_attr.html'

    def get_queryset(self):
        user = self.request.user

        try:
            device = get_devices().objects.get(device_fullname=\
                    self.kwargs['device_fullname'])
        except models.LdapDevice.DoesNotExist:
            return None

        if user.is_superuser or user.username == device.device_username:
            return [device]
        else:
            return None


device_attr = login_required(DeviceAttrView.as_view())

class DeviceListView(generic_views.ListView):
    """
    Displaying the list of devices
    """
    model = models.LdapDevice
    template_name = 'granadilla/device_list.html'

    def get_queryset(self):
        user = self.request.user
        if user.is_superuser:
            return get_devices().objects.all()
        else:
            return get_devices().objects.filter(device_username=user.username)


device_list = login_required(DeviceListView.as_view())


class GroupView(generic_views.DetailView):
    model = models.LdapGroup
    template_name = 'granadilla/group.html'
    printable = False
    slug_field = 'name'

    def get_context_data(self, **kwargs):
        ctxt = super(GroupView, self).get_context_data(**kwargs)
        ctxt.update({
            'printable': self.printable,
            'home': self.object.name == settings.GRANADILLA_USERS_GROUP,
            'group': self.object,
            'members': models.LdapUser.objects.filter(username__in=self.object.usernames),
        })
        return ctxt


group = login_required(GroupView.as_view())



class PrintableGroupView(GroupView):
    printable = True


"""
Display a printable list of users belonging to a group.
"""
group_print = login_required(PrintableGroupView.as_view())


class GroupsView(generic_views.ListView):
    """
    Display the list of groups.
    """
    model = models.LdapGroup
    template_name = 'granadilla/group_list.html'


groups = login_required(GroupsView.as_view())


@login_required
def photo(request, uid):
    now = time.time()
    max_age = 1800
    if not was_modified_since(request.META.get('HTTP_IF_MODIFIED_SINCE'), now - max_age):
        return HttpResponseNotModified()

    user = get_object_or_404(models.LdapUser, pk=uid)
    if not user.photo:
        return HttpResponseRedirect(granadilla_media('img/unknown.png'))
    response = HttpResponse()
    response['Cache-Control'] = 'max-age=%i' % max_age
    response['Content-Length'] = len(user.photo)
    response['Content-Type'] = 'image/jpeg'
    response['Last-Modified'] = http_date(now)
    response.write(user.photo)
    return response

def photo_delete(request, uid):
    user = get_object_or_404(models.LdapUser, pk=uid)
    if not can_write(request.user, user):
        raise PermissionDenied

    if request.method == 'POST':
        user.photo = ''
        user.save()
        return redirect_to(request, reverse(index))
    else:
        return render_to_response('granadilla/photo_delete.html', RequestContext(request, {
            'object': user,
        }))

@login_required
def user(request, uid):
    user = get_object_or_404(models.LdapUser, pk=uid)

    # set permissions
    can_edit = can_write(request.user, user)

    # handle form
    if request.method == 'POST':
        if not can_edit:
            raise PermissionDenied
        form = LdapUserForm(request.POST, request.FILES, instance=user)
        if form.is_valid():
            form.save()
            return redirect_to(request, reverse(index))
    else:
        form = LdapUserForm(instance=user)

    return render_to_response('granadilla/user.html', RequestContext(request, {
        'can_edit': can_edit,
        'object': user,
        'form': form,
    }))

@login_required
def user_card(request, uid):
    user = get_object_or_404(models.LdapUser, pk=uid)
    return user_vcard(user)
 



# -*- coding: utf-8 -*-
# 
# django-granadilla
# Copyright (C) 2009-2012 Bolloré telecom
# See AUTHORS file for a full list of contributors.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import base64
try:
    import hashlib
    md5_constructor = hashlib.md5
except ImportError:
    import md5
    md5_constructor = md5.new
import os
import time
import unicodedata

from .conf import settings
from django.utils.translation import ugettext_lazy as _

from ldapdb import models as ldap_models
from ldapdb.models import fields as ldap_fields



def normalise(str):
    nkfd_form = unicodedata.normalize('NFKD', unicode(str))
    return u"".join([c for c in nkfd_form if not unicodedata.combining(c)])


def hash_password(password):
    m = md5_constructor()
    m.update(password.encode('utf-8'))
    return "{MD5}" + base64.b64encode(m.digest())


class LdapAcl(ldap_models.Model):
    """
    Class for representing an LDAP ACL entry.
    """
    # LDAP meta-data
    base_dn = settings.GRANADILLA_ACLS_DN
    object_classes = ['groupOfNames']

    # groupOfNames
    name = ldap_fields.CharField(_("name"), db_column='cn', primary_key=True)
    members = ldap_fields.ListField(_("members"), db_column='member')

    def save(self):
        if not self.members:
            self.delete()
        else:
            super(LdapAcl, self).save()

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ('name',)
        verbose_name = _("access control list")
        verbose_name_plural = _("access control lists")


class LdapGroup(ldap_models.Model):
    """
    Class for representing an LDAP group entry.
    """
    # LDAP meta-data
    base_dn = settings.GRANADILLA_GROUPS_DN
    object_classes = ['posixGroup']

    # posixGroup
    gid = ldap_fields.IntegerField(_("identifier"), db_column='gidNumber', unique=True)
    name = ldap_fields.CharField(_("name"), db_column='cn', primary_key=True)
    usernames = ldap_fields.ListField(_("usernames"), db_column='memberUid')

    def __str__(self):
        return self.name

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ('name',)
        verbose_name = _("group")
        verbose_name_plural = _("groups")


class LdapContact(ldap_models.Model):
    """
    Class for representing an LDAP contact entry.
    """
    # LDAP meta-data
    object_classes = ['inetOrgPerson']

    # inetOrgPerson
    first_name = ldap_fields.CharField(_("first name"), db_column='givenName')
    last_name = ldap_fields.CharField(_("last name"), db_column='sn')
    full_name = ldap_fields.CharField(_("full name"), db_column='cn', primary_key=True)
    organization = ldap_fields.CharField(_("organization"), db_column='o', blank=True)
    email = ldap_fields.CharField(_("e-mail address"), db_column='mail', blank=True)
    phone = ldap_fields.CharField(_("phone"), db_column='telephoneNumber', blank=True)
    mobile_phone = ldap_fields.CharField(_("mobile phone"), db_column='mobile', blank=True)
    photo = ldap_fields.ImageField(_("photo"), db_column='jpegPhoto')
    postal_address = ldap_fields.CharField(_("postal address"), db_column='postalAddress', blank=True)

    def __unicode__(self):
        return self.full_name

    def last_name_initial(self):
        if len(self.last_name) > 0:
            return self.last_name[0].upper()
        else:
            return ''

    def save(self):
        self.full_name = "%s %s" % (self.first_name, self.last_name)
        super(LdapContact, self).save()

    class Meta:
        abstract = True
        ordering = ('last_name', 'first_name')
        verbose_name = _("contact")
        verbose_name_plural = _("contacts")


class LdapServiceAccount(ldap_models.Model):
    """Class for a Service account."""
    # LDAP meta-data
    base_dn = settings.GRANADILLA_SERVICES_DN
    object_classes = ['person', 'uidObject']

    username = ldap_fields.CharField(_("username"), db_column='uid', primary_key=True)
    first_name = ldap_fields.CharField(_("name (copy)"), db_column='sn', editable=False)
    last_name = ldap_fields.CharField(_("name (copy)"), db_column='cn', editable=False)
    password = ldap_fields.CharField(_("password"), db_column='userPassword')
    description = ldap_fields.CharField(_("description"), db_column='description')

    class Meta:
        verbose_name = _("service account")
        verbose_name_plural = _("service accounts")

    def __str__(self):
        return self.username

    def __unicode__(self):
        return self.username

    def set_password(self, password):
        self.password = hash_password(password)

    def save(self, *args, **kwargs):
        self.first_name = self.last_name = self.username
        super(LdapServiceAccount, self).save(*args, **kwargs)


class LdapUser(ldap_models.Model):
    """
    Class for representing an LDAP user entry.

    >>> q = LdapUser.objects.filter(username="foo")
    >>> q.query.where.as_sql()
    '(uid=foo)'

    >>> q = LdapUser.objects.filter(username__in=["foo", "bar"])
    >>> q.query.where.as_sql()
    '(|(uid=foo)(uid=bar))'
    """
    # LDAP meta-data
    base_dn = settings.GRANADILLA_USERS_DN
    object_classes = ['posixAccount', 'shadowAccount', 'inetOrgPerson']
    if settings.GRANADILLA_USE_SAMBA:
        object_classes.append('sambaSamAccount')

    # inetOrgPerson
    first_name = ldap_fields.CharField(_("first name"), db_column='givenName')
    last_name = ldap_fields.CharField(_("last name"), db_column='sn')
    full_name = ldap_fields.CharField(_("full name"), db_column='cn')
    email = ldap_fields.CharField(_("e-mail address"), db_column='mail', blank=True)
    phone = ldap_fields.CharField(_("phone"), db_column='telephoneNumber', blank=True)
    mobile_phone = ldap_fields.CharField(_("mobile phone"), db_column='mobile', blank=True)
    photo = ldap_fields.ImageField(_("photo"), db_column='jpegPhoto')

    # FIXME: this is a hack
    internal_phone = ldap_fields.CharField(_("internal phone"), db_column='roomNumber', blank=True)

    # posixAccount
    uid = ldap_fields.IntegerField(_("user id"), db_column='uidNumber', unique=True)
    group = ldap_fields.IntegerField(_("group id"), db_column='gidNumber')
    gecos =  ldap_fields.CharField(db_column='gecos')
    home_directory = ldap_fields.CharField(_("home directory"), db_column='homeDirectory')
    login_shell = ldap_fields.CharField(_("login shell"), db_column='loginShell', default=settings.GRANADILLA_USERS_SHELL)
    username = ldap_fields.CharField(_("username"), db_column='uid', primary_key=True)
    password = ldap_fields.CharField(_("password"), db_column='userPassword')

    # samba
    if settings.GRANADILLA_USE_SAMBA:
        samba_sid = ldap_fields.CharField(db_column='sambaSID')
        samba_lmpassword = ldap_fields.CharField(db_column='sambaLMPassword')
        samba_ntpassword = ldap_fields.CharField(db_column='sambaNTPassword')
        samba_pwdlastset = ldap_fields.IntegerField(db_column='sambaPwdLastSet')

    def defaults(self, key):
        if key == "email":
            email = "-".join(normalise(self.first_name).split(" "))
            email += "."
            email += "-".join(normalise(self.last_name).split(" "))
            email += "@"
            email += settings.GRANADILLA_MAIL_DOMAIN
            return email.lower()
        elif key == "full_name":
            return " ".join([self.first_name, self.last_name])
        elif key == "gecos":
            return normalise(self.full_name)
        elif key == "group":
            group = LdapGroup.objects.get(name=settings.GRANADILLA_USERS_GROUP)
            return group.gid
        elif key == "home_directory":
            return os.path.join(settings.GRANADILLA_USERS_HOME, self.username)
        elif key == "login_shell":
            return settings.GRANADILLA_USERS_SHELL
        raise Exception("No defaults for %s" % key)

    def __str__(self):
        return self.username

    def __unicode__(self):
        return self.full_name

    def set_password(self, password):
        self.password = hash_password(password)
        if settings.GRANADILLA_USE_SAMBA:
            import smbpasswd
            self.samba_ntpassword = smbpasswd.nthash(password)
            self.samba_lmpassword = smbpasswd.lmhash(password)
            self.samba_pwdlastset = int(time.time())

    def save(self):
        if settings.GRANADILLA_USE_SAMBA and not self.samba_sid:
            self.samba_sid = "%s-%i" % (settings.GRANADILLA_SAMBA_PREFIX, self.uid * 2 + 1000)
        super(LdapUser, self).save()
        
    class Meta:
        ordering = ('last_name', 'first_name')
        verbose_name = _("user")
        verbose_name_plural = _("users")


class LdapOrganizationalUnit(ldap_models.Model):
    """
    Class for representing an LDAP organization unit entry.
    """
    # LDAP meta-data
    base_dn = settings.GRANADILLA_BASE_DN
    object_classes = ['organizationalUnit']

    # organizationalUnit
    name = ldap_fields.CharField(_("name"), db_column='ou', primary_key=True)


class LdapExternalUser(ldap_models.Model):
    """
    An external user.
    """
    # LDAP meta-data
    base_dn = settings.GRANADILLA_EXTERNAL_USERS_DN
    object_classes = ['inetOrgPerson']

    # inetOrgPerson
    first_name = ldap_fields.CharField(_("first name"), db_column='givenName')
    last_name = ldap_fields.CharField(_("last name"), db_column='sn')
    full_name = ldap_fields.CharField(_("full name"), db_column='cn')
    email = ldap_fields.CharField(_("e-mail address"), db_column='mail', primary_key=True)

    def __str__(self):
        return self.username

    def __unicode__(self):
        return self.full_name

    class Meta:
        ordering = ('last_name', 'first_name')
        verbose_name = _("external user")
        verbose_name_plural = _("external users")

    def save(self, *args, **kwargs):
        self.full_name = u"%s %s" % (self.first_name, self.last_name)
        return super(LdapExternalUser, self).save(*args, **kwargs)


class LdapDevice(ldap_models.Model):
    """
    A device for the VPN.
    """
    # LDAP meta-data
    base_dn = settings.GRANADILLA_DEVICES_DN
    object_classes = ['device', 'simpleSecurityObject']

    # device
    device_fullname = ldap_fields.CharField(_("device fullname"), db_column='cn', primary_key=True)
    device_name = ldap_fields.CharField(_("device name"), db_column='description')
    device_owner = ldap_fields.CharField(_("device owner"), db_column='owner')
    device_username = ldap_fields.CharField(_("device username"), db_column='o')

    # simpleSecurityObject   
    password = ldap_fields.CharField(_("password"), db_column='userPassword')
 
    def __str__(self):
        return self.device_username

    def __unicode__(self):
        return self.device_name

    def check_password(self, password):
        return self.password == hash_password(password)

    def set_password(self, password):
        self.password = hash_password(password)

    def save(self, *args, **kwargs):
        return super(LdapDevice, self).save(*args, **kwargs)


if __name__ == "__main__":
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
# 
# django-granadilla
# Copyright (C) 2009-2012 Bolloré telecom
# See AUTHORS file for a full list of contributors.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


import appconf

from django.conf import settings


class GranadillaConf(appconf.AppConf):
    """Default settings for granadilla.

    Override in settings.py.
    """

    class Meta:
        prefix = 'granadilla'
        required = ['base_dn']

    # LDAP root
    BASE_DN = 'dc=example,dc=org'

    # Specific organizationalUnit for some components
    ACLS_DN = 'ou=groupacls,dc=example,dc=org'
    CONTACTS_DN = 'ou=contacts,dc=example,dc=org'
    GROUPS_DN = 'ou=groups,dc=example,dc=org'
    SERVICES_DN = 'ou=services,dc=example,dc=org'
    USERS_DN = 'ou=users,dc=example,dc=org'
    USE_ACLS = False

    # Homepage: "full company" group name
    USERS_GROUP = 'all'

    # Admin: list of admin group names
    ADMIN_GROUPS = []

    # Account settings
    USERS_HOME = '/home'
    USERS_SHELL = '/bin/bash'

    # Samba
    USE_SAMBA = False
    SAMBA_PREFIX = 'S-1-0-0'



# -*- coding: utf-8 -*-
# 
# django-granadilla
# Copyright (C) 2009-2012 Bolloré telecom
# See AUTHORS file for a full list of contributors.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django.contrib import admin

from .conf import settings
from . import models

class LdapAclAdmin(admin.ModelAdmin):
    exclude = ['dn', 'members']

if settings.GRANADILLA_USE_ACLS:
    admin.site.register(models.LdapAcl, LdapAclAdmin)


class LdapGroupAdmin(admin.ModelAdmin):
    exclude = ['dn', 'usernames']
    list_display = ['name', 'gid']
    search_fields = ['name']

admin.site.register(models.LdapGroup, LdapGroupAdmin)


class LdapUserAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {
            'fields': ('first_name', 'last_name', 'full_name'),
        }),
        ('Posix account', {
            'fields': ('username', 'uid', 'group', 'gecos', 'home_directory', 'login_shell')
        }),
        ('Personal info', {
            'classes': ('collapse',),
            'fields': ('email', 'phone', 'mobile_phone', 'internal_phone'),
        }),
    )
    list_display = ['username', 'first_name', 'last_name', 'email', 'uid']
    search_fields = ['first_name', 'last_name', 'full_name', 'username']

admin.site.register(models.LdapUser, LdapUserAdmin)


class LdapServiceAccountAdmin(admin.ModelAdmin):
    exclude = ['dn']
    list_display = ['username', 'first_name', 'last_name', 'description']
    search_fields = list_display

admin.site.register(models.LdapServiceAccount, LdapServiceAccountAdmin)


class LdapDeviceAdmin(admin.ModelAdmin):
    exclude = ['dn']
    list_display = ['device_fullname', 'device_name', 'device_owner']
    search_fields = list_display

admin.site.register(models.LdapDevice, LdapDeviceAdmin)



"""
Django settings for granadilla_webapp project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""

from __future__ import unicode_literals

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import getconf
import os
BASE_DIR = os.path.dirname(__file__)
CHECKOUT_DIR = os.path.dirname(BASE_DIR)

config = getconf.ConfigGetter('granadilla',
    [
        '/etc/granadilla/settings.ini',
        os.path.join(CHECKOUT_DIR, 'local_settings.ini'),
    ],
)

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/
env = config.get('env', 'dev')
assert env in ('dev', 'prod'), "Invalid environment %s" % env

# SECURITY WARNING: keep the secret key used in production secret!
if env == 'dev':
    _default_secret_key = 'Dev only!!'
else:
    _default_secret_key = ''

SECRET_KEY = config.get('django.secret_key', _default_secret_key)

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = config.getbool('dev.debug', env == 'dev')

TEMPLATE_DEBUG = DEBUG

ALLOWED_HOSTS = config.getlist('django.allowed_hosts')


# Application definition

INSTALLED_APPS = (
    'granadilla',
    'granadilla_webapp.web',

    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'granadilla_webapp.urls'

WSGI_APPLICATION = 'granadilla_webapp.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': config.get('db.engine', 'django.db.backends.sqlite3'),
        'NAME': config.get('db.name', os.path.join(BASE_DIR, 'db.sqlite3')),
        'HOST': config.get('db.host'),
        'PORT': config.get('db.port'),
        'USER': config.get('db.user'),
        'PASSWORD': config.get('db.password'),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/

STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(BASE_DIR, 'static')


# LDAP
LDAPDB_BIND_DN = config.get('ldap.webapp_bind_dn', 'uid=test,dc=example,dc=org')
LDAPDB_BIND_PASSWORD = config.get('ldap.webapp_bind_pw')
LDAPDB_SERVER_URI = config.get('ldap.server', 'ldaps://ldaps.example.org')

# Base DN for LDAP database.
GRANADILLA_BASE_DN = config.get('granadilla.base_dn', 'dc=example,dc=org')

# The organizationalUnit for groups
GRANADILLA_GROUPS_OU = config.get('granadilla.groups_ou', 'ou=groups')
GRANADILLA_GROUPS_DN = '%s,%s' % (GRANADILLA_GROUPS_OU, GRANADILLA_BASE_DN)
# The organizationalUnit for users
GRANADILLA_USERS_OU = config.get('granadilla.users_ou', 'ou=users')
GRANADILLA_USERS_DN = '%s,%s' % (GRANADILLA_USERS_OU, GRANADILLA_BASE_DN)
# The organizationalUnit for external users
GRANADILLA_EXTERNAL_USERS_OU = config.get('granadilla.external_users_ou', 'ou=external_users')
GRANADILLA_EXTERNAL_USERS_DN = '%s,%s' % (GRANADILLA_EXTERNAL_USERS_OU, GRANADILLA_BASE_DN)
# The organizationalUnit for devices
GRANADILLA_DEVICES_OU = config.get('granadilla.devices_ou', 'ou=external_users')
GRANADILLA_DEVICES_DN = '%s,%s' % (GRANADILLA_DEVICES_OU, GRANADILLA_BASE_DN)
# The organizationalUnit for services
GRANADILLA_SERVICES_OU = config.get('granadilla.services_ou', 'ou=services')
GRANADILLA_SERVICES_DN = '%s,%s' % (GRANADILLA_SERVICES_OU, GRANADILLA_BASE_DN)
# The organizationalUnit for contacts
GRANADILLA_CONTACTS_OU = config.get('granadilla.contacts_ou', 'ou=contacts')
GRANADILLA_CONTACTS_DN = '%s,%s' % (GRANADILLA_CONTACTS_OU, GRANADILLA_BASE_DN)
# The organizationalUnit for acls
GRANADILLA_ACLS_OU = config.get('granadilla.acls_ou', 'ou=acls')
GRANADILLA_ACLS_DN = '%s,%s' % (GRANADILLA_ACLS_OU, GRANADILLA_BASE_DN)
GRANADILLA_USE_ACLS = config.getbool('granadilla.use_acls', False)

# Domain to automatically generate e-mail addresses for new users.
GRANADILLA_MAIL_DOMAIN = config.get('granadilla.mail_domain', 'example.org')
# The home folder for user accounts
GRANADILLA_USERS_HOME = config.get('granadilla.users_home', '/home')
# The "base" group which is displayed in the index view.
GRANADILLA_USERS_GROUP = config.get('granadilla.users_group', 'test')
# The 'admin' groups
GRANADILLA_ADMIN_GROUPS = config.getlist('granadilla.admin_groups')

# Whether to use samba
GRANADILLA_USE_SAMBA = config.getbool('granadilla.use_samba', False)
# Samba SID prefix
GRANADILLA_SAMBA_PREFIX = config.get('granadilla.samba_prefix', 'S-1-0-0')

# URL from which Granadilla's static media are served.
GRANADILLA_MEDIA_PREFIX = STATIC_URL



# -*- coding: utf-8 -*-
# 
# django-granadilla
# Copyright (C) 2009-2011 Bolloré telecom
# See AUTHORS file for a full list of contributors.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os.path

from django.conf import settings
from django.template import Library
from django.utils.translation import ugettext as _

import granadilla

register = Library()

def granadilla_title():
    try:
        return settings.GRANADILLA_TITLE
    except:
        return _("Phonebook")
register.simple_tag(granadilla_title)

def granadilla_version():
    return "<a href=\"%s\" title=\"Granadilla %s\">%s</a>" % (granadilla.__url__, granadilla.__version__, granadilla_title())
register.simple_tag(granadilla_version)

def granadilla_media(medium):
    """
    Returns the path to static media.
    """
    try:
        prefix = settings.GRANADILLA_MEDIA_PREFIX
    except:
        try:
            prefix = os.path.join(settings.STATIC_URL, 'granadilla')
        except:
            prefix = os.path.join(settings.MEDIA_URL, 'granadilla')
    return os.path.join(prefix, medium)
register.simple_tag(granadilla_media)

def field_value(field):
    if not field.form.is_bound:
        data = field.form.initial.get(field.name, field.field.initial)
        if callable(data):
            data = data()
    else:
        data =field.data
    return data
register.simple_tag(field_value)




from django.conf.urls import patterns, include, url

from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    # accounts
    (r'^accounts/login/$', 'django.contrib.auth.views.login'),
    (r'^accounts/logout/$', 'django.contrib.auth.views.logout'),

    # admin interface
    url(r'^admin/', include(admin.site.urls)),

    # granadilla
    (r'', include('granadilla.urls')),
)



# -*- coding: utf-8 -*-
# 
# django-granadilla
# Copyright (C) Bolloré telecom, Polyconseil
# See AUTHORS file for a full list of contributors.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""
Granadilla

@author: Aymeric Augustin <aymeric.augustin@bolloretelecom.eu>
@author: Jeremy Lainé <jeremy.laine@bolloretelecom.eu>
@author: Raphaël Barrois <raphael.barrois@polyconseil.fr
"""

__docformat__ = 'epytext en'

__url__ = 'https://github.com/Polyconseil/granadilla'
__copyright__ = "(C) Bolloré Telecom, Polyconseil"
__license__ = 'GPL'
__license_long__ = """
 Copyright (C) Bolloré Telecom, Polyconseil
  All rights reserved."""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import find_packages, setup

from granadilla.version import VERSION


def read(filename):
    with open(filename) as fp:
        return fp.read()


setup(
    name="granadilla",
    version=VERSION,
    author="Polyconseil Sysadmin Team",
    author_email="sysadmin+granadilla@polyconseil.fr",
    description="An opinionated LDAP frontend.",
    license="GPL",
    keywords=['granadilla', 'ldap', 'directory', 'admin'],
    url="https://github.com/Polyconseil/granadilla",
    packages=find_packages(),
    long_description=read('README.rst'),
    install_requires=[
        # Django core
        'Django>=1.7,<1.8',

        # Databases
        'django-ldapdb',

        # Configuration
        'django-appconf',
        'getconf>=1.2.0',

        # Passwords
        'smbpasswd',
        'zxcvbn',

        # Command line
        'colorama',
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Environment :: Console",
        "Framework :: Django",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 2.7",
        "Topic :: System :: Systems Administration :: Authentication/Directory :: LDAP",
    ],
    include_package_data=True,
)



"""Trivial vcard generator."""

import re


class Field(object):
    SINGLE = 1
    MULTI = 2
    MULTI_NESTED = 3

    def __init__(self, prefix, subtype='', cardinality=SINGLE):
        self.prefix = prefix
        self.subtype = subtype
        self.cardinality = cardinality

    def _escape_one(self, value):
        value = re.sub(r'([,;\\])', r'\\\1', value)
        value = value.replace('\n', '\\n')
        return value

    def _escape(self, value):
        if self.cardinality == self.SINGLE:
            return self._escape_one(value)
        elif self.cardinality == self.MULTI:
            return ';'.join(self._escape_one(v) for v in value)
        else:
            assert self.cardinality == self.MULTI_NESTED
            return ';'.join(
                ','.join(self._escape_one(subv) for subv in v)
                for v in value
            )

    def filled(self, value):
        if self.cardinality == self.SINGLE:
            return bool(value)
        elif self.cardinality == self.MULTI:
            return any(v for v in value)
        else:
            assert self.cardinality == self.MULTI_NESTED
            return any(any(subv for subv in v) for v in value)

    def render(self, value):
        if self.subtype:
            subtype_txt = ';TYPE=%s' % self.subtype
        else:
            subtype_txt = ''
        return '%s%s:%s' % (self.prefix, subtype_txt, self._escape(value))


class VCard(object):

    ALL_FIELDS = {
        'src': Field('SOURCE'),
        'kind': Field('KIND'),
        'email': Field('EMAIL', subtype='INTERNET'),
        'full_name': Field('FN'),
        'names': Field('N', cardinality=Field.MULTI_NESTED),
        'cell': Field('TEL', subtype='CELL'),
        'phone': Field('TEL', subtype='VOICE'),
        'org': Field('ORG', cardinality=Field.MULTI_NESTED),
        'photo': Field('PHOTO'),
    }

    def __init__(self):
        self.data = {}

    def __getitemm__(self, key):
        return self.data[key]

    def __setitem__(self, key, value):
        assert key in self.ALL_FIELDS
        self.data[key] = value

    def render_bytes(self):
        inner_lines = []
        for key, field in self.ALL_FIELDS.items():
            if key not in self.data:
                continue
            value = self.data[key]
            if not field.filled(value):
                continue
            inner_lines.append(field.render(value))

        lines = [
            'BEGIN:VCARD',
            'VERSION:3.0',
        ] + sorted(inner_lines) + [
            'END:VCARD',
        ]
        return '\r\n'.join(lines).encode('utf-8')



# -*- coding: utf-8 -*-
# 
# django-granadilla
# Copyright (C) 2009-2012 Bolloré telecom
# See AUTHORS file for a full list of contributors.
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django.conf.urls import patterns, url

urlpatterns = patterns('',
    (r'^$', 'granadilla.views.index'),
    (r'^contacts/$', 'granadilla.views.contact_list'),
    (r'^contacts/create/$', 'granadilla.views.contact_create'),
    (r'^contacts/(?P<contact_id>[^/]+)/$', 'granadilla.views.contact'),
    (r'^contacts/(?P<contact_id>[^/]+)/card/$', 'granadilla.views.contact_card'),
    (r'^contacts/(?P<contact_id>[^/]+)/delete/$', 'granadilla.views.contact_delete'),
    url(r'^devices/$', 'granadilla.views.device_list', name="device_list"),
    (r'^devices/create/$', 'granadilla.views.device_create'),
    (r'^devices/(?P<device_fullname>[^/]+)/$', 'granadilla.views.device_attr'),
    (r'^devices/(?P<device_fullname>[^/]+)/password/$', 'granadilla.views.device_password'),
    (r'^devices/(?P<device_fullname>[^/]+)/delete/$', 'granadilla.views.device_delete'),
    (r'^groups/$', 'granadilla.views.groups'),
    (r'^group/(?P<slug>.*)/print/$', 'granadilla.views.group_print'),
    (r'^group/(?P<slug>.*)/$', 'granadilla.views.group'),
    (r'^user/(?P<uid>.*)/card/$', 'granadilla.views.user_card'),
    (r'^user/(?P<uid>.*)/photo/$', 'granadilla.views.photo'),
    (r'^user/(?P<uid>.*)/photo/delete/$', 'granadilla.views.photo_delete'),
    (r'^user/(?P<uid>.*)/$', 'granadilla.views.user'),
)

